@@ -2,7 +2,7 @@
  * vinagre-window.c
  * This file is part of vinagre
  *
- * Copyright (C) 2007 - Jonh Wendell <wendell@bani.com.br>
+ * Copyright (C) 2007,2008 - Jonh Wendell <wendell@bani.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
  
 #ifdef HAVE_CONFIG_H
@@ -36,7 +34,7 @@
 #include "vinagre-fav.h"
 #include "vinagre-prefs-manager.h"
 #include "vinagre-utils.h"
-#include "vinagre-favorites.h"
+#include "vinagre-bookmarks.h"
 #include "vinagre-ui.h"
 
 #include "vinagre-window-private.h"
@@ -65,7 +63,7 @@ vinagre_window_finalize (GObject *object)
     }
 
 
-  vinagre_favorites_finalize ();
+  vinagre_bookmarks_finalize ();
 
   G_OBJECT_CLASS (vinagre_window_parent_class)->finalize (object);
 }
@@ -75,11 +73,19 @@ vinagre_window_delete_event (GtkWidget   *widget,
 			     GdkEventAny *event)
 {
   VinagreWindow *window = VINAGRE_WINDOW (widget);
+  GtkClipboard  *cb;
+
+  cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 
   if (window->priv->signal_notebook > 0)
     g_signal_handler_disconnect (window->priv->notebook,
 				 window->priv->signal_notebook);
 
+  if (window->priv->signal_clipboard > 0)
+    g_signal_handler_disconnect (cb,
+				 window->priv->signal_clipboard);
+
+  vinagre_window_close_all_tabs (window);
   gtk_main_quit ();
 
   if (GTK_WIDGET_CLASS (vinagre_window_parent_class)->delete_event)
@@ -105,6 +111,7 @@ vinagre_window_show_hide_controls (VinagreWindow *window)
       gtk_widget_hide (window->priv->statusbar);
 
       gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->priv->notebook), FALSE);
+      gtk_notebook_set_show_border (GTK_NOTEBOOK (window->priv->notebook), FALSE);
     }
   else
     {
@@ -120,6 +127,7 @@ vinagre_window_show_hide_controls (VinagreWindow *window)
         gtk_widget_show_all (window->priv->statusbar);
 
       gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->priv->notebook), TRUE);
+      gtk_notebook_set_show_border (GTK_NOTEBOOK (window->priv->notebook), TRUE);
     }
 }
 
@@ -129,7 +137,7 @@ vinagre_window_state_event_cb (GtkWidget *widget,
 {
   VinagreWindow *window;
 
-  if (event->changed_mask != GDK_WINDOW_STATE_FULLSCREEN)
+  if ((event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) == 0)
     return FALSE;
 
   window = VINAGRE_WINDOW (widget);
@@ -177,7 +185,7 @@ vinagre_window_key_press_cb (GtkWidget   *widget,
       case GDK_F11:
 	if (window->priv->active_tab)
 	  vinagre_window_toggle_fullscreen (window);
-	break;
+	return FALSE;
     }
 
   return GTK_WIDGET_CLASS (vinagre_window_parent_class)->key_press_event (widget, event);
@@ -199,16 +207,125 @@ vinagre_window_class_init (VinagreWindowClass *klass)
   g_type_class_add_private (object_class, sizeof(VinagreWindowPrivate));
 }
 
+static void
+menu_item_select_cb (GtkItem       *proxy,
+		     VinagreWindow *window)
+{
+  GtkAction *action;
+  char *message;
+
+  action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+  g_return_if_fail (action != NULL);
+
+  g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+  if (message)
+    {
+      gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
+			  window->priv->tip_message_cid, message);
+      g_free (message);
+    }
+}
+
+static void
+menu_item_deselect_cb (GtkItem       *proxy,
+                       VinagreWindow *window)
+{
+  gtk_statusbar_pop (GTK_STATUSBAR (window->priv->statusbar),
+		     window->priv->tip_message_cid);
+}
+
+static void
+connect_proxy_cb (GtkUIManager  *manager,
+                  GtkAction     *action,
+                  GtkWidget     *proxy,
+                  VinagreWindow *window)
+{
+  if (GTK_IS_MENU_ITEM (proxy))
+    {
+       g_signal_connect (proxy, "select",
+			 G_CALLBACK (menu_item_select_cb), window);
+       g_signal_connect (proxy, "deselect",
+			 G_CALLBACK (menu_item_deselect_cb), window);
+    }
+}
+
+static void
+disconnect_proxy_cb (GtkUIManager  *manager,
+                     GtkAction     *action,
+                     GtkWidget     *proxy,
+                     VinagreWindow *window)
+{
+  if (GTK_IS_MENU_ITEM (proxy))
+    {
+      g_signal_handlers_disconnect_by_func
+		(proxy, G_CALLBACK (menu_item_select_cb), window);
+      g_signal_handlers_disconnect_by_func
+		(proxy, G_CALLBACK (menu_item_deselect_cb), window);
+    }
+}
+
+static void
+activate_recent_cb (GtkRecentChooser *action, VinagreWindow *window)
+{
+  VinagreConnection *conn;
+
+  conn = vinagre_connection_new_from_string (gtk_recent_chooser_get_current_uri (action));
+  vinagre_cmd_open_bookmark (window, conn);
+
+  vinagre_connection_free (conn);
+}
+
+static void update_recent_connections (VinagreWindow *window)
+{
+  VinagreWindowPrivate *p = window->priv;
+
+  g_return_if_fail (p->recent_action_group != NULL);
+
+  if (p->recents_menu_ui_id != 0)
+    gtk_ui_manager_remove_ui (p->manager, p->recents_menu_ui_id);
+
+  p->recents_menu_ui_id = gtk_ui_manager_new_merge_id (p->manager);
+
+  gtk_ui_manager_add_ui (p->manager,
+			 p->recents_menu_ui_id,
+			 "/MenuBar/MachineMenu/FileRecentsPlaceholder",
+			 "recent_connections",
+			 "recent_connections",
+			 GTK_UI_MANAGER_MENUITEM,
+			 FALSE);
+}
+
+static void
+recent_manager_changed (GtkRecentManager *manager,
+			VinagreWindow     *window)
+{
+  update_recent_connections (window);
+}
+
 static void
 create_menu_bar_and_toolbar (VinagreWindow *window, 
 			     GtkWidget     *main_box)
 {
-  GtkActionGroup *action_group;
-  GtkUIManager *manager;
-  GError *error = NULL;
+  GtkActionGroup   *action_group;
+  GtkUIManager     *manager;
+  GError           *error = NULL;
+  GtkRecentManager *recent_manager;
+  GtkRecentFilter  *filter;
+  GtkAction        *action;
 
   manager = gtk_ui_manager_new ();
   window->priv->manager = manager;
+
+  /* show tooltips in the statusbar */
+  g_signal_connect (manager,
+		    "connect-proxy",
+		    G_CALLBACK (connect_proxy_cb),
+		    window);
+  g_signal_connect (manager,
+		   "disconnect-proxy",
+		    G_CALLBACK (disconnect_proxy_cb),
+		    window);
+
   gtk_window_add_accel_group (GTK_WINDOW (window),
 			      gtk_ui_manager_get_accel_group (manager));
 
@@ -228,6 +345,9 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
   g_object_unref (action_group);
   window->priv->always_sensitive_action_group = action_group;
 
+  action = gtk_action_group_get_action (action_group, "MachineConnect");
+  g_object_set (action, "is_important", TRUE, NULL);
+
   action_group = gtk_action_group_new ("VinagreWindowActions");
   gtk_action_group_set_translation_domain (action_group, NULL);
   gtk_action_group_add_actions (action_group,
@@ -239,6 +359,26 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
   g_object_unref (action_group);
   window->priv->action_group = action_group;
 
+  action = gtk_action_group_get_action (action_group, "MachineClose");
+  g_object_set (action, "is_important", TRUE, NULL);
+
+  /* Machine connected actions */
+  action_group = gtk_action_group_new ("VinagreWindowMachineConnectedActions");
+  gtk_action_group_set_translation_domain (action_group, NULL);
+  gtk_action_group_add_actions (action_group,
+				vinagre_machine_connected_menu_entries,
+				G_N_ELEMENTS (vinagre_machine_connected_menu_entries),
+				window);
+
+  gtk_ui_manager_insert_action_group (manager, action_group, 0);
+  g_object_unref (action_group);
+  window->priv->machine_connected_action_group = action_group;
+
+  action = gtk_action_group_get_action (action_group, "ViewFullScreen");
+  g_object_set (action, "is_important", TRUE, NULL);
+  action = gtk_action_group_get_action (action_group, "MachineTakeScreenshot");
+  g_object_set (action, "is_important", TRUE, NULL);
+
   /* now load the UI definition */
   gtk_ui_manager_add_ui_from_file (manager, vinagre_utils_get_ui_xml_filename (), &error);
   if (error != NULL)
@@ -247,10 +387,10 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
       g_error_free (error);
     }
 
-  /* list of open documents menu */
-  action_group = gtk_action_group_new ("DocumentsListActions");
+  /* Bookmarks */
+  action_group = gtk_action_group_new ("BookmarksActions");
   gtk_action_group_set_translation_domain (action_group, NULL);
-  window->priv->favorites_list_action_group = action_group;
+  window->priv->bookmarks_list_action_group = action_group;
   gtk_ui_manager_insert_action_group (manager, action_group, 0);
   g_object_unref (action_group);
 
@@ -268,6 +408,38 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
 		      FALSE,
 		      FALSE,
 		      0);
+
+  /* Recent connections */
+  window->priv->recent_action = gtk_recent_action_new ("recent_connections",
+						     _("_Recent connections"),
+						       NULL, NULL);
+  g_signal_connect (window->priv->recent_action,
+		    "item-activated",
+		    G_CALLBACK (activate_recent_cb),
+		    window);
+
+  action_group = gtk_action_group_new ("VinagreRecentConnectionsActions");
+  gtk_action_group_set_translation_domain (action_group, NULL);
+
+  gtk_ui_manager_insert_action_group (manager, action_group, 0);
+  g_object_unref (action_group);
+  window->priv->recent_action_group = action_group;
+  gtk_action_group_add_action (window->priv->recent_action_group,
+			       window->priv->recent_action);
+
+  filter = gtk_recent_filter_new ();
+  gtk_recent_filter_add_group (filter, "vinagre");
+  gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (window->priv->recent_action),
+				 filter);
+
+  update_recent_connections (window);
+
+  recent_manager = gtk_recent_manager_get_default ();
+  g_signal_connect (recent_manager,
+		    "changed",
+		    G_CALLBACK (recent_manager_changed),
+		    window);
+
 }
 
 static void
@@ -279,6 +451,9 @@ set_machine_menu_sensitivity (VinagreWindow *window)
 
   active = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook)) > 0;
   gtk_action_group_set_sensitive (window->priv->action_group, active);
+
+  active = window->priv->machines_connected > 0;
+  gtk_action_group_set_sensitive (window->priv->machine_connected_action_group, active);
 }
 
 static void
@@ -299,7 +474,7 @@ fav_panel_activated (VinagreFav        *fav,
   g_return_if_fail (VINAGRE_IS_FAV (fav));
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
 
-  vinagre_cmd_open_favorite (window, conn);
+  vinagre_cmd_open_bookmark (window, conn);
 }
 
 static void
@@ -310,9 +485,9 @@ fav_panel_selected (VinagreFav        *fav,
   GtkAction *action1, *action2;
 
   action1 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "FavoritesDel");
+					 "BookmarksDel");
   action2 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "FavoritesEdit");
+					 "BookmarksEdit");
 
   if (window->priv->fav_conn_selected)
     {
@@ -336,34 +511,35 @@ fav_panel_selected (VinagreFav        *fav,
 }
 
 void
-vinagre_window_update_favorites_list_menu (VinagreWindow *window)
+vinagre_window_update_bookmarks_list_menu (VinagreWindow *window)
 {
   VinagreWindowPrivate *p = window->priv;
-  GList *actions, *l, *favs;
-  gint n, i;
-  guint id;
-  VinagreConnection *conn;
-  gchar *action_name, *action_label;
-  GtkAction *action;
+  GList                *actions, *l, *favs;
+  gint                  n, i;
+  guint                 id;
+  VinagreConnection    *conn;
+  gchar                *action_name, *action_label;
+  GtkAction            *action;
+  gchar                *name, *tooltip;
 
-  g_return_if_fail (p->favorites_list_action_group != NULL);
+  g_return_if_fail (p->bookmarks_list_action_group != NULL);
 
-  if (p->favorites_list_menu_ui_id != 0)
-    gtk_ui_manager_remove_ui (p->manager, p->favorites_list_menu_ui_id);
+  if (p->bookmarks_list_menu_ui_id != 0)
+    gtk_ui_manager_remove_ui (p->manager, p->bookmarks_list_menu_ui_id);
 
-  actions = gtk_action_group_list_actions (p->favorites_list_action_group);
+  actions = gtk_action_group_list_actions (p->bookmarks_list_action_group);
   for (l = actions; l != NULL; l = l->next)
     {
       g_signal_handlers_disconnect_by_func (GTK_ACTION (l->data),
-					    G_CALLBACK (vinagre_cmd_favorites_open),
+					    G_CALLBACK (vinagre_cmd_bookmarks_open),
 					    window);
-      gtk_action_group_remove_action (p->favorites_list_action_group,
+      gtk_action_group_remove_action (p->bookmarks_list_action_group,
 				      GTK_ACTION (l->data));
     }
     g_list_free (actions);
 
-  /* Get the favorites from file */
-  favs = vinagre_favorites_get_all ();
+  /* Get the bookmarks from file */
+  favs = vinagre_bookmarks_get_all ();
   n = g_list_length (favs);
   i = 0;
 
@@ -372,30 +548,34 @@ vinagre_window_update_favorites_list_menu (VinagreWindow *window)
   while (favs)
     {
       conn = (VinagreConnection *) favs->data;
+      name = vinagre_connection_best_name (conn);
 
       action_name = g_strdup_printf ("Fav_%d", i);
+      /* Translators: This is server:port, a statusbar tooltip when mouse is over a bookmark item on menu */
+      tooltip = g_strdup_printf (_("Open %s:%d"), conn->host, conn->port);
       action_label = vinagre_utils_escape_underscores (
-			vinagre_connection_best_name (conn),
-			-1);
+		     name,
+		     -1);
       action = gtk_action_new (action_name,
 			       action_label,
-			       NULL,
+			       tooltip,
 			       NULL);
+      g_object_set (G_OBJECT (action), "icon-name", "application-x-vnc", NULL);
       g_object_set_data_full (G_OBJECT (action),
 			      "conn",
 			      conn,
 			      (GDestroyNotify) vinagre_connection_free);
-      gtk_action_group_add_action (p->favorites_list_action_group,
+      gtk_action_group_add_action (p->bookmarks_list_action_group,
 				   GTK_ACTION (action));
 
       g_signal_connect (action,
 			"activate",
-			G_CALLBACK (vinagre_cmd_favorites_open),
+			G_CALLBACK (vinagre_cmd_bookmarks_open),
 			window);
 
       gtk_ui_manager_add_ui (p->manager,
 			     id,
-			     "/MenuBar/FavoritesMenu/FavoritesList",
+			     "/MenuBar/BookmarksMenu/BookmarksList",
 			     action_name, action_name,
 			     GTK_UI_MANAGER_MENUITEM,
 			     FALSE);
@@ -403,6 +583,8 @@ vinagre_window_update_favorites_list_menu (VinagreWindow *window)
       g_object_unref (action);
       g_free (action_name);
       g_free (action_label);
+      g_free (name);
+      g_free (tooltip);
 
       favs = favs->next;
       i++;
@@ -410,7 +592,7 @@ vinagre_window_update_favorites_list_menu (VinagreWindow *window)
 
   g_list_free (favs);
 
-  p->favorites_list_menu_ui_id = id;
+  p->bookmarks_list_menu_ui_id = id;
 }
 
 static void
@@ -448,27 +630,18 @@ init_widgets_visibility (VinagreWindow *window)
   GtkAction *action;
   gboolean visible;
 
-  /* Remove and Edit favorites starts disabled */
+  /* Remove and Edit bookmarks starts disabled */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"FavoritesDel");
+					"BookmarksDel");
   gtk_action_set_sensitive (action, FALSE);
 
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"FavoritesEdit");
-  gtk_action_set_sensitive (action, FALSE);
-
-  /* TODO: Implement these commands */
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"HelpContents");
-  gtk_action_set_sensitive (action, FALSE);
-
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"EditPreferences");
+					"BookmarksEdit");
   gtk_action_set_sensitive (action, FALSE);
 
   /* fav panel visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"ViewFavorites");
+					"ViewBookmarks");
   visible = vinagre_prefs_manager_get_side_pane_visible ();
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
@@ -539,6 +712,11 @@ create_statusbar (VinagreWindow *window,
 {
   window->priv->statusbar = gtk_statusbar_new ();
 
+  window->priv->generic_message_cid = gtk_statusbar_get_context_id
+	(GTK_STATUSBAR (window->priv->statusbar), "generic_message");
+  window->priv->tip_message_cid = gtk_statusbar_get_context_id
+	(GTK_STATUSBAR (window->priv->statusbar), "tip_message");
+
   gtk_box_pack_end (GTK_BOX (main_box),
 		    window->priv->statusbar,
 		    FALSE, 
@@ -546,22 +724,24 @@ create_statusbar (VinagreWindow *window,
 		    0);
 }
 
-static void 
+void
 vinagre_window_set_title (VinagreWindow *window)
 {
-  gchar *title;
+  gchar *title, *name;
 
   if (window->priv->active_tab == NULL)
     {
-      gtk_window_set_title (GTK_WINDOW (window), "vinagre");
+      gtk_window_set_title (GTK_WINDOW (window), g_get_application_name ());
       return;
     }
 
+  name = vinagre_connection_best_name (vinagre_tab_get_conn (VINAGRE_TAB (window->priv->active_tab)));
   title = g_strdup_printf ("%s - %s",
-			   vinagre_connection_best_name (vinagre_tab_get_conn (VINAGRE_TAB (window->priv->active_tab))),
-			   "vinagre");
+			   name,
+			   g_get_application_name ());
   gtk_window_set_title (GTK_WINDOW (window), title);
   g_free (title);
+  g_free (name);
 }
 
 static void
@@ -574,6 +754,9 @@ vinagre_window_page_removed (GtkNotebook   *notebook,
 
   window->priv->active_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->priv->notebook), gtk_notebook_get_current_page (GTK_NOTEBOOK (window->priv->notebook)));
 
+  if (!window->priv->active_tab && window->priv->fullscreen)
+    vinagre_window_toggle_fullscreen (window);
+
   vinagre_window_set_title (window);
   set_machine_menu_sensitivity (window);
 }
@@ -614,7 +797,7 @@ vinagre_window_switch_page (GtkNotebook     *notebook,
 static void
 create_notebook (VinagreWindow *window)
 {
-  window->priv->notebook = vinagre_notebook_new ();
+  window->priv->notebook = vinagre_notebook_new (window);
 
   gtk_paned_pack2 (GTK_PANED (window->priv->hpaned), 
   		   window->priv->notebook,
@@ -640,20 +823,47 @@ create_notebook (VinagreWindow *window)
   gtk_widget_show (window->priv->notebook);
 }
 
+static void
+vinagre_window_clipboard_cb (GtkClipboard *cb, GdkEvent *event, VinagreWindow *window)
+{
+  gchar *text;
+
+  text = gtk_clipboard_wait_for_text (cb);
+  if (text) {
+    if (window->priv->active_tab)
+      vinagre_tab_paste_text (VINAGRE_TAB (window->priv->active_tab), text);
+    g_free (text);
+  }
+
+}
+
+static void
+vinagre_window_init_clipboard (VinagreWindow *window)
+{
+  GtkClipboard *cb;
+
+  cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+  window->priv->signal_clipboard = g_signal_connect (cb,
+						     "owner-change",
+						     G_CALLBACK (vinagre_window_clipboard_cb),
+						     window);
+}
+
 static void
 vinagre_window_init (VinagreWindow *window)
 {
   GtkWidget *main_box;
 
-  gtk_window_set_default_icon_name ("gnome-remote-desktop");
+  gtk_window_set_default_icon_name ("vinagre");
 
   window->priv = VINAGRE_WINDOW_GET_PRIVATE (window);
   window->priv->active_tab = NULL;
   window->priv->fav_conn_selected = NULL;
   window->priv->fullscreen = FALSE;
   window->priv->signal_notebook = 0;
+  window->priv->machines_connected = 0;
 
-  vinagre_favorites_init ();
+  vinagre_bookmarks_init ();
 
   main_box = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), main_box);
@@ -686,7 +896,8 @@ vinagre_window_init (VinagreWindow *window)
   init_widgets_visibility (window);
   set_machine_menu_sensitivity (window);
 
-  vinagre_window_update_favorites_list_menu (window);
+  vinagre_window_update_bookmarks_list_menu (window);
+  vinagre_window_init_clipboard (window);
 }
 
 GtkWidget *
@@ -739,7 +950,7 @@ vinagre_window_new ()
 {
   return g_object_new (VINAGRE_TYPE_WINDOW,
 		       "type",      GTK_WINDOW_TOPLEVEL,
-		       "title",     "Vinagre",
+		       "title",     g_get_application_name (),
 		       NULL); 
 }
 
@@ -774,6 +985,30 @@ vinagre_window_get_toolbar (VinagreWindow *window)
   return window->priv->toolbar;
 }
 
+GtkWidget *
+vinagre_window_get_menubar (VinagreWindow *window)
+{
+  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+
+  return window->priv->menubar;
+}
+
+GtkActionGroup *
+vinagre_window_get_main_action (VinagreWindow *window)
+{
+  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+
+  return window->priv->action_group;
+}
+
+GtkActionGroup *
+vinagre_window_get_sensitive_action (VinagreWindow *window)
+{
+  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+
+  return window->priv->always_sensitive_action_group;
+}
+
 GtkWidget *
 vinagre_window_get_fav_panel (VinagreWindow *window)
 {
@@ -791,6 +1026,14 @@ vinagre_window_close_active_tab	(VinagreWindow *window)
 			    VINAGRE_TAB (window->priv->active_tab));
 }
 
+VinagreTab *
+vinagre_window_get_active_tab	(VinagreWindow *window)
+{
+  g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
+
+  return VINAGRE_TAB (window->priv->active_tab);
+}
+
 GtkUIManager *
 vinagre_window_get_ui_manager (VinagreWindow *window)
 {
@@ -799,3 +1042,25 @@ vinagre_window_get_ui_manager (VinagreWindow *window)
   return window->priv->manager;
 }
 
+void
+_vinagre_window_add_machine_connected (VinagreWindow *window)
+{
+  g_return_if_fail (VINAGRE_IS_WINDOW (window));
+
+  window->priv->machines_connected++;
+  set_machine_menu_sensitivity (window);
+}
+
+void
+_vinagre_window_del_machine_connected (VinagreWindow *window)
+{
+  g_return_if_fail (VINAGRE_IS_WINDOW (window));
+
+  if (window->priv->machines_connected > 0)
+    {
+      window->priv->machines_connected--;
+      set_machine_menu_sensitivity (window);
+    }
+}
+
+/* vim: ts=8 */
